
/*
 * $Id: StatIntvlRec.cc,v 1.8.6.5 1999/12/07 15:30:20 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#include "polygraph.h"

#include <math.h>

#include "ILog.h"
#include "OLog.h"
#include "StatIntvlRec.h"

#include "gadgets.h"


StatIntvlRec::StatIntvlRec() {
	restart();
}

void StatIntvlRec::restart() {
	theXactLvl.restart();
	theOpenLvl.restart();
	theEstbLvl.restart();

	theConnLifeTm.reset();
	theConnUseCnt.reset();

	theIdealHR.reset();
	theRealHR.reset();
	theChbR.reset();

	theHitRespTm.reset();
	theMissRespTm.reset();

	theFill.reset();
	theXactCnt = theXactErrCnt = 0;

	theScanLen.reset();

	theDuration = Time();
}

OLog &StatIntvlRec::store(OLog &log) const {
	Assert(theDuration >= 0);
	return log
		<< theXactLvl << theOpenLvl << theEstbLvl
		<< theConnLifeTm << theConnUseCnt
		<< theIdealHR << theRealHR << theChbR
		<< theHitRespTm << theMissRespTm
		<< theFill
		<< theXactCnt
		<< theXactErrCnt
		<< theDuration
		<< theScanLen
		;
}

ILog &StatIntvlRec::load(ILog &log) {
	return log
		>> theXactLvl >> theOpenLvl >> theEstbLvl
		>> theConnLifeTm >> theConnUseCnt
		>> theIdealHR >> theRealHR >> theChbR
		>> theHitRespTm >> theMissRespTm
		>> theFill
		>> theXactCnt
		>> theXactErrCnt
		>> theDuration
		>> theScanLen
		;
}

double StatIntvlRec::errRatio() const {
	return Ratio(theXactErrCnt, complCnt() + theXactErrCnt);
}

double StatIntvlRec::errPercent() const {
	return Percent(theXactErrCnt, complCnt() + theXactErrCnt);
}

// XXX: we should have Rate type, not double
double StatIntvlRec::reqRate() const {
	return theDuration > 0 ?
		Ratio(theXactLvl.incCnt(), theDuration.secd()) : -1;
}

double StatIntvlRec::repRate() const {
	return theDuration > 0 ?
		Ratio(theXactLvl.decCnt(), theDuration.secd()) : -1;
}

AggrStat StatIntvlRec::respTime() const {
	AggrStat h(theHitRespTm);
	h += theMissRespTm;
	return h;
}

BigSize StatIntvlRec::repSz() const {
	return theRealHR.xactSz(); // hits + misses
}

Size StatIntvlRec::repSzMean() const {
	if (const int x = theRealHR.xacts())
		return (theRealHR.xactSz()/x).byte();
	return -1;
}

Time StatIntvlRec::scanTime() const {
	return theDuration > 0 && theScanLen.count() ?
		theDuration / theScanLen.count() : Time();
}

Time StatIntvlRec::scanStepTime() const {
	return theDuration > 0 && theScanLen.sum() > 0 ?
		theDuration / theScanLen.sum() : Time();
}

void StatIntvlRec::concat(const StatIntvlRec &r) {
	if (theDuration >= 0)
		theDuration += r.theDuration;
	join(r);
}

void StatIntvlRec::merge(const StatIntvlRec &s) {
	join(s);
}

void StatIntvlRec::join(const StatIntvlRec &s) {
	Assert(s.theDuration >= 0);
	if (theDuration < 0)
		theDuration = s.theDuration;

	theXactLvl.mergeWith(s.theXactLvl);
	theOpenLvl.mergeWith(s.theOpenLvl);
	theEstbLvl.mergeWith(s.theEstbLvl);

	theConnLifeTm += s.theConnLifeTm;
	theConnUseCnt += s.theConnUseCnt;

	theIdealHR += s.theIdealHR;
	theRealHR += s.theRealHR;
	theChbR += s.theChbR;

	theHitRespTm += s.theHitRespTm;
	theMissRespTm += s.theMissRespTm;
	
	theFill += s.theFill;
	theXactCnt += s.theXactCnt;
	theXactErrCnt += s.theXactErrCnt;

	theScanLen += s.theScanLen;
}
