
/*
 * $Id: StatsSampleMgr.cc,v 1.1.2.2 1999/10/23 23:13:54 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#include "polygraph.h"

#include <iostream.h>
#include <iomanip.h>

#include "StatsSampleMgr.h"
#include "Xaction.h"
#include "PolyOLog.h"
#include "LogComment.h"

#include "StatsSampleSym.h"
#include "StatsSampleCfg.h"

#include "polyLogTags.h"

StatsSampleMgr::StatsSampleMgr(): theStartCnt(0) {
}

StatsSampleMgr::~StatsSampleMgr() {
	while (theCfgs.count()) delete theCfgs.pop();
}

void StatsSampleMgr::configure(const Array<StatsSampleSym*> &cfgs) {
	theCfgs.stretch(cfgs.count());
	for (int i = 0; i < cfgs.count(); ++i) {
		StatsSampleCfg *cfg = new StatsSampleCfg;
		cfg->name = cfgs[i]->name();
		cfg->start = cfgs[i]->start();
		cfgs[i]->capacity(cfg->capacity);

		if (cfg->start < 0 || cfg->capacity <= 0) {
			cerr << cfgs[i]->loc() << "malformed StatsSample entry ";
			if (cfg->name)
				cerr << " named " << cfg->name;
			cerr << endl;
			exit(-2);
		}

		theCfgs.append(cfg);
	}
}

void StatsSampleMgr::start() {
	schedSample();
}

void StatsSampleMgr::wakeUp(const Alarm &a) {
	AlarmUser::wakeUp(a);
	startSample();
	schedSample();
}

void StatsSampleMgr::startSample() {
	if (theStartCnt < theCfgs.count()) {
		if (Xaction::TheSampleDebt > 0) {
			Comment(3) << "warning: stats samples overlap; "
				<< Xaction::TheSampleDebt << " entries still uncollected" 
				<< endc;
		}
		const StatsSampleCfg &cfg = *theCfgs[theStartCnt];
		Xaction::TheSampleDebt += cfg.capacity;

		Comment(3) << "fyi: started sampling ";
		if (cfg.name)
			Comment << "`" << cfg.name << "' ";
		Comment << '(' << cfg.capacity << " entries)";
		Comment << endc;

		theStartCnt++;

		(*TheSmplOLog) << bege(lgStatsSmplBeg)
			<< cfg.name << cfg.start << cfg.capacity << ende;
	}
}

void StatsSampleMgr::schedSample() {
	if (theStartCnt < theCfgs.count())
		sleepTill(Clock::TheStartTime + theCfgs[theStartCnt]->start);
}

void StatsSampleMgr::reportCfg(ostream &os) const {
	if (!theCfgs.count()) {
		os << "<no stats samples are scheduled>" << endl;
		return;
	}

	// header
	os 
		<< setw(16) << "sample"
		<< ' ' << setw(8) << "capacity"
		<< ' ' << setw(8) << "start"
		<< endl;

	for (int i = 0; i < theCfgs.count(); ++i) {
		const StatsSampleCfg &cfg = *theCfgs[i];
		os
			<< setw(16) << (cfg.name ? (const char*)cfg.name : "<anonymous>")
			<< ' ' << setw(8) << cfg.capacity
			<< ' ' << setw(8) << cfg.start
			<< endl;
	}
	
}
