
/*
 * $Id: ConnHash.h,v 1.1.2.3 1999/10/11 17:42:52 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#ifndef PG_CONNECTION_HASH_H
#define PG_CONNECTION_HASH_H

#include "Array.h"
#include "Farm.h"
#include "Connection.h"

// address -> connection map

class NetAddr;
class ConnHashItem;

typedef ConnHashItem **ConnHashPos;

struct ConnHashQueueItem: public QueueItem {
	Connection *conn;

	ConnHashQueueItem(): conn(0) {}
	void reset() { conn = 0; }
};

class ConnHashItem {
	public:
		ConnHashItem();

		void reset();

		bool vacant() const;
		bool matches(const NetAddr &addr) const;

		Queue<ConnHashQueueItem> &queue() { return theQueue; }
		ConnHashItem *collision() { return theCollision; }
		ConnHashPos nextPos() { return &theCollision; }

	protected:
		Queue<ConnHashQueueItem> theQueue;
		ConnHashItem *theCollision;
};

class ConnHash {
	public:
		ConnHash(int ccAddrMax = 0);

		void ccAddrMax(int aMax);

		void add(Connection *conn);
		void del(Connection *conn);
		Connection *delAt(ConnHashPos pos);
		bool find(const NetAddr &addr);
		bool find(const NetAddr &addr, ConnHashPos &pos);

	protected:
		int hash(const NetAddr &addr) const;
		void delAt(ConnHashPos pos, ConnHashQueueItem *qi);

	protected:
		static ObjFarm<ConnHashItem> TheItems;
		static ObjFarm<ConnHashQueueItem> TheQueueItems;
		Array<ConnHashItem*> theHash;
};

#endif
