
/*
 * $Id: ServerRep.h,v 1.2.4.5 1999/11/29 01:21:01 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#ifndef PG_SERVER_REPRESENTATIVE_H
#define PG_SERVER_REPRESENTATIVE_H

// client side "representative" of an origin server 
// holds client side knowledge about a server

#include "Ring.h"
#include "NetAddr.h"
#include "UniqId.h"

class OLog;

class ServerRep {
	public:
		static void ConfigureNids(int prefetchLmt, int perReqMax);

	public:
		ServerRep(const NetAddr &anAddr);

		const NetAddr &addr() const { return theAddr; }
		const UniqId &id() const { return theId; }

		bool haveNewOids() const { return theNewOids.count(); }
		int workSetCap() const { return theWorkSetCap; }
		int accesses() const { return theAccCount; }

		int oidShortage(); // changes number of "pending" oid reqs

		int newOid();
		int secOid() const { return theSecOid; }
		int maxNewOidCap() const { return theNewOids.capacity(); }
		int pendCount() const { return thePendCnt; }

		void noteNewOids(int startOid, int askCnt, int gotCnt);
		void syncSrvInfo(const UniqId &aSrvId, int aSecOid);
		void noteNoNewOids(); // client ran out of new oids
		void noteAccess() { theAccCount++; }

		void freezeWorkSetCap();

		void store(OLog &log) const;

	protected:
		static int TheNewOidsPrefetchDepth; // control new oid prefetching
		static int TheNewOidsPerReqMax;     //  algorithm

	protected:
		NetAddr theAddr;
		UniqId theId;
		Ring<int> theNewOids;
		int theSecOid;     // our idea of server's "secure" oid
		int theAccCount;   // accesses to this server
		int theWorkSetCap; // number of oids in server's work set

	private:
		int thePendCnt; // outstanding reqs for new oids
};

#endif
