
/*
 * $Id: XactStatRec.h,v 1.1.2.2 1999/10/23 23:11:41 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#ifndef PG_XACTION_RECORD_CLASS_H
#define PG_XACTION_RECORD_CLASS_H

#include "UniqId.h"
#include "Oid.h"

// summarizes single transaction statistics

class XactStatRec {
	public:
		XactStatRec();		

		Time startTime() const { return theStartTime; }
		Time queueTime() const { return Time::Msec(theQueueMsec); }
		Time lifeTime() const { return Time::Msec(theLifeMsec); }

		inline void times(Time start, Time queue, Time life);

		void store(OLog &ol) const;
		void load(ILog &il);

	public:
		UniqId theId;
		int theOwnerSeqId; // xaction owner
		int theConnSeqId;     // connection id

		Time theStartTime; // started
		int theQueueMsec;  // time spent in a queue (msec)
		int theLifeMsec;   // (msec)

		int theEmbedCount;  // number of embedded object in the reply
		Size theRepSize;    // reply size
		OidRec theOidRec;   // various details about the transfered object
};

/* inlined methods */

inline
void XactStatRec::times(Time start, Time queue, Time life) {
	theStartTime = start;
	theQueueMsec = queue >= 0 ? (start - queue).msec() : -1;
	theLifeMsec = life.msec();
}

#endif
