
/*
 * $Id: pglSyms.h,v 1.5.6.1 1999/11/23 00:18:09 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#ifndef PGL_SYMBOLS_H
#define PGL_SYMBOLS_H

#include "TokenSym.h"
#include "PglRec.h"
#include "PglExprSym.h"

// various language symbols


// template to define simple symbols
#define SimpleSynSym(Type) \
	struct Type##Sym: public SynSym { \
		Type##Sym(): SynSym(#Type) {} \
		Type##Sym(const char *aType): SynSym(aType) {} \
	};

SimpleSynSym(Code)
SimpleSynSym(StatementSeq)
SimpleSynSym(Statement)
SimpleSynSym(SimpleStatement)
SimpleSynSym(BlockStatement)
SimpleSynSym(DeclStatement)
SimpleSynSym(ExprStatement)
SimpleSynSym(InitDecl)
SimpleSynSym(Assignment)
SimpleSynSym(PartAssign)
SimpleSynSym(BlockOpen)
SimpleSynSym(BlockClose)


class SynSymTblItem;

struct PureDeclSym: public SynSym {
	SynSymTblItem *theDecl;
	PureDeclSym(SynSymTblItem *aDecl): SynSym("PureDecl"), theDecl(aDecl) {}
	// does not detroy declaration
};

// "empty" or "no op" expression -- does not change LHS value
class NopExprSym: public ExpressionSym {
	public:
		NopExprSym(): ExpressionSym("NopExpr") {}
};

// "undefined" or "null" expression -- undefines LHS value
class UndefOpExprSym: public ExpressionSym {
	public:
		UndefOpExprSym();

		virtual bool isA(const String &type) const;
};

// function call
class CallSym: public ExpressionSym {
	public:
		CallSym(): ExpressionSym("Call") {}
};

#define TypedTokenSym(Type) \
	struct Type##Sym: public TokenSym { \
		Type##Sym(const String &s): TokenSym(s) { type(#Type); } \
		virtual SynSym *clone(const String &type) const { \
			if (isA(type)) return new Type##Sym(spelling()); \
			else return TokenSym::clone(type); } \
	};

TypedTokenSym(ObjName);
TypedTokenSym(KnownCtxName);
TypedTokenSym(CtxName);
TypedTokenSym(Type);
TypedTokenSym(UnOper);
TypedTokenSym(BinOper);

#endif
