
/*
 * $Id: AlarmClock.h,v 1.6.4.1 1999/11/02 04:22:24 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#ifndef ALARM_CLOCK_H
#define ALARM_CLOCK_H

#include "Heap.h"
#include "Clock.h"

class AlarmUser;

// one time alarm
class Alarm {
	public:
		Alarm(): theSleeper(0) {}
		Alarm(Time aTime, AlarmUser *aSleeper): theTime(aTime), theSleeper(aSleeper) {}

		operator void*() const { return theSleeper ? (void*)-1 : 0; }
		Time time() const { return theTime; }

		const AlarmUser *sleeper() const { return theSleeper; }

		void set(Time aTime, AlarmUser *aSleeper);
		void ring();
		void snooze();
		void clear() { theSleeper = 0; }

		// these are used by AlarmQueue to order the alarms
		bool operator <(const Alarm &a) const { return theTime < a.theTime; }
		bool operator >(const Alarm &a) const { return theTime > a.theTime; }

	protected:
		Time theTime;
		AlarmUser *theSleeper;
};

// somebody who can wait for an alarm
// alarm count is used to check if pending alarms must be cleared 
// before object destruction
class AlarmUser {
	public:
		AlarmUser();
		virtual ~AlarmUser();

		bool alarmWaiting() const { return thePendAlarmCnt; }

		virtual void wakeUp(const Alarm &alarm) = 0;  // but has a body!
		virtual void snooze(const Alarm &alarm); // the alarm got cancelled

	protected:
		void sleepTill(Time time);
		void sleepFor(Time delay);

	protected:
		int thePendAlarmCnt;
};

// a sequence of ordered Alarms
typedef Heap<Alarm> AlarmQueue;


// alarm clock that can handle unlimited number of alarms
class AlarmClock: public Clock {
	public:
		AlarmClock();
		~AlarmClock() {}

		bool clear() const { return theQueue.empty(); }
		Time timeLeft() const { return theTimeLeft; }

		void update(Time curTime); // rings if needed
		void setAlarm(const Alarm &alarm);
		void cancelAll(const AlarmUser *sleeper);

	protected:
		void ring(); // wakes up ready sleepers
		void noteNewRingTime(Time time);

	protected:
		AlarmQueue theQueue;
		Time theRingTime;
		Time theTimeLeft;
};

extern AlarmClock TheAlarmClock;

#endif
