
/*
 * $Id: Assert.h,v 1.3 1999/09/07 17:48:35 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#ifndef XSTD_ASSERT_H
#define XSTD_ASSERT_H

#include <stdlib.h>
#include <errno.h>

class ostream;

// our version of assert(3), mostly for portability purposes
#define Assert(cond) ((cond) ? (void)0 : Abort(__FILE__, __LINE__, #cond))

// same as Assert but calls Exit instead of Abort
#define Check(cond) ((cond) ? (void)0 : Exit(__FILE__, __LINE__, #cond))

// logs current error to cerr and exits if cond fails
#define Must(cond) ((cond) ? (void)0 : (Complain(__FILE__, __LINE__), ::exit(errno)))

// logs current error to cerr if cond fails
#define Should(cond) ((cond) ? true : (Complain(__FILE__, __LINE__), false))


// perror(3) wrapper (handy as a stream manipulator)
// note: str() method can be overwritten to support custom error messages
class error {
	public:
		error(int anErrNo): theErrNo(anErrNo) {}
		virtual ~error() {}

		int errNo() const { return theErrNo; }
		virtual const char *str() const;
		ostream &print(ostream &os) const;

	protected:
		int theErrNo;
};

inline 
ostream &operator <<(ostream &os, const error &e) { return e.print(os); }


// handy for temporary debugging
#define here __FILE__ << ':' << __LINE__ << ": "

/* internal functions used by macros above */

// logs current err to cerr
extern void Complain(const char *fname, int lineno);

// aborts program execution and generates coredump
extern void Abort(const char *fname, int lineno, const char *cond);

// aborts program execution without coredump
extern void Exit(const char *fname, int lineno, const char *cond);

#endif
